﻿/**
* Encrypts and decrypts values in memory to stop hackers.
* Michael Grundvig, Electrotank, Inc. 05/18/2007
* http://www.electrotank.com
* @Version 1.0
* Available for free use anywhere and in any way you want.
* Of course, no promises are made to the capability, reliability, or usefulness of this code
*/
package com.electrotank.util {
	
	import com.meychi.ascrypt3.TEA;
	
	public class MemoryCrypto {
		
		// Define some internal enum variables
		private static var INCREMENT:int = 0;
		private static var DECREMENT:int = 1;
		private static var MULTIPLY:int = 2;
		private static var DIVIDE:int = 3;
		
		// Define the various state variables
		private static var crypto:TEA = null;
		private static var key:String = null;
		private static var dataHolder:Object = null;
		private static var initialized:Boolean = false;
		
		public static function getValue(name:String):Number {
			initIfNeeded();
			
			// Default out our results 
			var results:Number = 0;
			
			// If the value hasn't been set, 
			if(!isValueSet(name)) {
				
				// go ahead and set it to 0
				setValue(name, results);
				
			} else {
				
				// If here, then the value has already been set, get it
				results = new Number(crypto.decrypt(dataHolder[name], key));
			}
			
			// Return the results
			return results;
		}
		
		public static function setValue(name:String, value:Number):Number {
			initIfNeeded();
			
			// Set the new value
			dataHolder[name] = crypto.encrypt(value.toString(), key);
			
			// Return the results
			return value;
		}
		
		public static function incrementValue(name:String, amount:Number):Number {
			
			// Process and return the results
			return processValue(name, amount, INCREMENT);
		}
		
		public static function decrementValue(name:String, amount:Number):Number {
			
			// Process and return the results
			return processValue(name, amount, DECREMENT);
		}
		
		public static function multiplyValue(name:String, amount:Number):Number {
			
			// Process and return the results
			return processValue(name, amount, MULTIPLY);
		}
		
		public static function divideValue(name:String, amount:Number):Number {
			
			// Prevent a divide by zero problem
			if(amount == 0) {
				throw new Error("Amount can't be zero when dividing!");
			}
			
			// Process and return the results
			return processValue(name, amount, DIVIDE);
		}
		
		private static function processValue(name:String, amount:Number, operation:int):Number {
			initIfNeeded();
			
			// Get the value
			var tempValue:Number = getValue(name);
			
			// Perform the requested operation
			switch(operation) {
				case INCREMENT:
					tempValue += amount;
					break;
				case DECREMENT:
					tempValue -= amount;
					break;
				case MULTIPLY:
					tempValue *= amount;
					break;
				case DIVIDE:
					tempValue /= amount;
					break;
			}
			
			// Return the results
			setValue(name, tempValue);
			
			// Return the results
			return tempValue;
		}
		
		
		public static function isValueSet(name:String):Boolean {
			initIfNeeded();
			
			// See if a variable by this name is already established
			var results:Boolean = false;
			if(dataHolder[name] != null) {
				results = true;
			}
			
			// Return the results
			return results;
		}
		
		public static function reset():void {
			
			// Set the initialization state back to 0
			initialized = false;
			
			// Re-initialize everything
			initIfNeeded();
		}
		
		private static function initIfNeeded():void {
			
			// If we haven't yet been initialized
			if(!initialized) {
				
				// Create the first part of the key
				var firstNumber:Number = Math.random();
				var firstKey:String = new String(firstNumber);
				firstKey = firstKey.substr(2, firstKey.length - 2);
				
				// Create the second part of the key
				var secondNumber:Number = Math.random();
				var secondKey:String = new String(secondNumber);
				secondKey = secondKey.substr(2, secondKey.length - 2);
				
				// Concatenate for the final key
				key = firstKey + secondKey;
			
				// Create a reference to the crypto engine
				crypto = new TEA();
				
				// Create the data holder object
				dataHolder = new Object();
				
				// Flag that we have now been initialized
				initialized = true;
				
			} // end initialized if-statement
			
		} // end initIfNeeded method
		
	} // end class declaration
	
} // end package declaration